/*
 * USB iPhone module
 *
 * Copyright (C) 2007 Greg Kroah-Hartman <gregkh@suse.de>
 * Copyright (C) 2007 Matt Colyer <matt@colyer.name>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, version 2.
 *
 */

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/usb.h>

#define APPLE_VENDOR   0x05ac
#define IPHONE         0x1290
#define IPODTOUCH      0x1291
#define IPHONE3G       0x1292
#define IPODTOUCH2G    0x1293

static int debug;

#ifdef dbg
#undef dbg
#endif
#define dbg(dev, format, arg...)                               \
       if (debug)                                              \
               dev_printk(KERN_DEBUG , dev , format , ## arg)

static struct usb_device_id id_table [] = {
       { USB_DEVICE(APPLE_VENDOR, IPHONE) },
       { USB_DEVICE(APPLE_VENDOR, IPODTOUCH) },
       { USB_DEVICE(APPLE_VENDOR, IPHONE3G) },
       { USB_DEVICE(APPLE_VENDOR, IPODTOUCH2G) },
       { },    /* Terminating entry */
};
MODULE_DEVICE_TABLE(usb, id_table);

static int select_configuration(struct usb_device *udev)
{
       int retval;

       dbg(&udev->dev, "Calling set_configuration\n");
       retval = usb_driver_set_configuration(udev, 3);
       if (retval) {
               dev_err(&udev->dev, "Set Configuration failed :%d.\n", retval);
               goto exit;
       }

exit:
       return retval;
}

static int iphone_probe(struct usb_interface *intf,
                       const struct usb_device_id *id)
{
       struct usb_device *udev = interface_to_usbdev(intf);

       if (udev->actconfig->desc.bConfigurationValue == 3) {
               dbg(&udev->dev, "Configuration changed");
               return -ENODEV;
       }

       /* turn the power on */
       select_configuration(udev);

       /* we don't really want to bind to the device, userspace programs can
        * handle the syncing just fine, so get outta here. */
       return -ENODEV;
}

static void iphone_disconnect(struct usb_interface *intf)
{
}

static struct usb_driver iphone_driver = {
       .name =         "iphone",
       .probe =        iphone_probe,
       .disconnect =   iphone_disconnect,
       .id_table =     id_table,
};

static int __init iphone_init(void)
{
       return usb_register(&iphone_driver);
}

static void __exit iphone_exit(void)
{
       usb_deregister(&iphone_driver);
}

module_init(iphone_init);
module_exit(iphone_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Matt Colyer <matt@colyer.name>");
module_param(debug, bool, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(debug, "Debug enabled or not");
